 //Some C++ notes to Mark Lewin
//1. I have used mostly the same names as you did
//2. This program uses the system dialog maintenance code,
//   which does not support using accelerators and so the keys ar L,S&Esc
//3. I have used code that dynamically allocates the space required for data
//4. A very quick tutorial on C++ is below
//---------------------------------------------------------------------------------------------------------------------
//All coding lines (except compound statements) require a semicolon as the line/statement ending indicator
// "//"is a form of comment and is basically the same as ' in VB (ignore the rest of the line)
/*This text is also commented out*/
//All strings are automatically terminated with NULL (0x00 or \0) so "a" is two bytes long and "zz" is three bytes long

#include <windows.h>//For windows functions
#include <stdio.h>//Use C stream I/O lib for file input
#include <io.h>//Use low level (byte basis) I/O lib for file output (quicker with bin files)
#include <fcntl.h>//defines for low level I/O
#include <sys/stat.h>//defines for low level I/O
#include "resource.h"//Resource ids - menu & dlg & controls & accelerators

//Some data
char InputFile[_MAX_PATH] = "";
char OutputFile[_MAX_PATH] = "";
BYTE* df3Array = NULL;
short XDim = 0;
short YDim = 0;
short ZDim = 0;

//Saved process id
HINSTANCE hInst;

//Prototypes
void LoadFile(HWND hw);
void SaveFile(HWND hw);
//Prototype for a procedure that handles the messages recieved by the dialog box
BOOL CALLBACK df3MakerProc(
	HWND hw,  // handle to dialog box
	UINT m,     // message
	WPARAM w, // first message parameter(long int)
	LPARAM l  // second message parameter(long int)
);

//Program entry point-ignore parameters but hInst is like a process identification number
int WINAPI WinMain(HINSTANCE inst,HINSTANCE hPrevInst, LPSTR lpCmd, int nCmdShow ){

	hInst = inst;//Save process id

	//Bring up the dialog box ( sending messages to df3Proc then DefDlgProc then df3MakerProc) via a Win32 function used for this purpose
	//Notice 'Call' from VB is not req'd
	//MAKEINTRESOURCE coverts the resource id to a form used by DialogBox
	//process id, resource dlg box, owner window, prodecure for handling messages
	DialogBox(hInst,MAKEINTRESOURCE(IDD_DF3_MAKER),NULL,df3MakerProc);
	return 0;
}

//The following functions are simple conversions from VB -> C++
void Infomnu_Click(){

	//"\n" represents a new line character & has the value 13
	//The compiler automatically concatenates text strings separated by whitespace
	char MsgTxt[] = 
		"df3 Maker File Input Information\n"
		"\n"
		"df3 Maker requires input data from a comma delimited ASCII file.\n"
		"The default extention for this file is txt, but any file can be\n"
		"opened. The first three numbers in the input file must contain the\n"
		"number of bins (referred to as voxels in the POV-Ray documentation)\n"
		"for the x, y, and z axes. These dimensions must be positive integers.\n"
		"After this, integer data ranging from 0 to 255, in order of\n"
		"increasing x, y and z follow. Below is an example of a 4x3x2 input file.\n"
		"\n"
		"4,3,2\n"
		"0,100,100,0\n"
		"50,255,255,50\n"
		"0,100,100,0\n"
		"0,50,50,0\n"
		"25,100,100,25\n"
		"0,25,25,0\n"
		"\n"
		"Common errors include not delimiting properly, not including enough\n"
		"data, or incorrectly assigning the axis sizes.\n";

	//Call a Win32 function to show the message box
	//		owner window, Message, title, type
	MessageBox(NULL, MsgTxt, "File Input Format", MB_OK);
}

void Loadmnu_Click(HWND hw){
	LoadFile(hw);
}

void Quitmnu_Click(HWND hw){
	delete [] df3Array;//Delete the data
	EndDialog (hw, 0);//Close the dialog box & return 0 from the function DialogBox
}

void LoadFile(HWND hw){
	FILE* FileNum;
	int i;
	int j;
	int k;
	int ReadData;
	char FileTitle[_MAX_PATH] = "";
	
	//Show the open file dialog
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hw;
	ofn.hInstance = NULL;
	ofn.lpstrFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = InputFile;
	ofn.nMaxFile = _MAX_PATH;
	ofn.lpstrFileTitle = FileTitle;
	ofn.nMaxFileTitle = _MAX_PATH;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle = NULL;
	ofn.Flags = OFN_READONLY ;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = NULL;
	ofn.lCustData = 0L;
	ofn.lpfnHook = NULL;
	ofn.lpTemplateName = NULL;
	if (GetOpenFileName (&ofn)){
		FileNum = fopen(InputFile,"r");//Open the file or reading
		if(FileNum!=NULL){
			fscanf(FileNum," %d , %d , %d \n", &XDim, &YDim, &ZDim);
			if(XDim>0 && YDim>0 && ZDim>0 ){
				if(df3Array)delete [] df3Array;//Deallocate the old mem
				df3Array = (BYTE*)new BYTE[XDim*YDim*ZDim];//Allocate the new memory
				if(df3Array){//If successful in this allocation
					int p = 0;//Position in file
					for(i = 0; i < ZDim; i++){
						for(j = 0; j < YDim; j++){
							for(k = 0; k < XDim; k++){
								if( 0 == fscanf(FileNum," %d ,",&ReadData) ){
									delete [] df3Array;//Delete the data
									df3Array = NULL;//Make sure the pointer doesn't go out into space
									goto EH2;//Show the error to the user
								}
								//Make sure its in the right range
								if(ReadData < 0) ReadData = 0;
								if(ReadData > 255) ReadData = 255;
								df3Array[p] = ReadData;//Stick it in the data
								p++;//Increment position
							}//k loop
							fscanf(FileNum," \n ");//Get newlines at the end of a row
						}//j loop
					}//i loop
				}//if successful allocation
				else goto EH2;//Show the error to the user
			}//Sizes in the right range
			else goto EH2;
		}//File opened properly
		else goto EH2;
		fclose(FileNum);//Close the file
		char dim[50] = "";
		SetDlgItemText(hw,IDC_FILE,FileTitle);//Show the file name
		//Show the dimensions
		sprintf(dim,"%d",XDim);
		SetDlgItemText(hw,IDC_XDIM,dim);
		sprintf(dim,"%d",YDim);
		SetDlgItemText(hw,IDC_YDIM,dim);
		sprintf(dim,"%d",ZDim);
		SetDlgItemText(hw,IDC_ZDIM,dim);
		//Enable the Save button
		EnableWindow(GetDlgItem(hw,IDM_SAVE_DF3),TRUE);
	}//User didn't press cancel
	return;

EH2://Error handler
	fclose(FileNum);//Close the file
	MessageBox (hw,"There was an error in the input file!\nCheck it's formatting.","df3maker",MB_OK);//Tell the user
	SetDlgItemText(hw,IDC_FILE,"Error!");//Set the file name to Error!
	//Set the dimensions to zero
	XDim = YDim = ZDim = 0;
	char dim[2] = "0";//Small enhancement - set the sizes to "0" on error
	SetDlgItemText(hw,IDC_XDIM,dim);
	SetDlgItemText(hw,IDC_YDIM,dim);
	SetDlgItemText(hw,IDC_ZDIM,dim);
	EnableWindow(GetDlgItem(hw,IDM_SAVE_DF3),FALSE);//Disable the Save button
	return;
}//Loadfile

void SaveFile(HWND hw){

	//Show the save dialog
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof (OPENFILENAME);
	ofn.hwndOwner = hw;
	ofn.hInstance = NULL;
	ofn.lpstrFilter = "df3 Files (*.df3)\0*.df3\0";
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = OutputFile;
	ofn.nMaxFile = _MAX_PATH;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle = NULL;
	ofn.Flags = OFN_HIDEREADONLY ;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = "df3";
	ofn.lCustData = 0L;
	ofn.lpfnHook = NULL;
	ofn.lpTemplateName = NULL;
	if (GetSaveFileName (&ofn)){
		int FileNum;
		FileNum = _open(OutputFile,_O_BINARY|_O_WRONLY|_O_CREAT,_S_IWRITE);//Create\Open the file
		if(FileNum!=-1){//Created\Opened the file properly
			//Used to output the sizes in big-endian format
			BYTE
				HBx = HIBYTE(XDim),
				LBx = LOBYTE(XDim),
				HBy = HIBYTE(YDim),
				LBy = LOBYTE(YDim),
				HBz = HIBYTE(ZDim),
				LBz = LOBYTE(ZDim);
			//Write the sizes
			_write(FileNum,&HBx,1);
			_write(FileNum,&LBx,1);
			_write(FileNum,&HBy,1);
			_write(FileNum,&LBy,1);
			_write(FileNum,&HBz,1);
			_write(FileNum,&LBz,1);
			//Write the Data 
			_write(FileNum,df3Array,XDim*YDim*ZDim);
			//Close the file
			_close(FileNum);
			//Tell the user
			MessageBox(hw, "File Saved!", "df3maker",MB_OK);
		}//if opened properly
	}//User didn't press cancel
}//Save file

void Savemnu_Click(HWND hw){
	if(df3Array)//Small enhancement - only save if data has been got properly
		SaveFile(hw);
	else
		EnableWindow(GetDlgItem(hw,IDM_SAVE_DF3),FALSE);//Disable the Save button
}

void Splashmnu_Click(){

	char MsgTxt[] =
		"df3 Maker\n"
		"By Mark Lewin (rewritten in C++ by Pabs)\n"
		"\n"
		"A small program that will write df3 files\n"
		"(POV-Ray media density files) from data\n"
		"contained in a delimited text file.\n"
		"See About...Input file format for more\n"
		"information regarding input specifics.\n"
		"\n"
		"(c) Mark Lewin, 14-6-2000\n";

	MessageBox(NULL, MsgTxt, "df3 Maker", MB_OK);

}

//This is the secondary message handler (primary is the system dialog handler)
//CALLBACK indicates that this procedure is called by windows
BOOL CALLBACK df3MakerProc(HWND hw, UINT m, WPARAM w, LPARAM l){
	//Switch is like SELECT CASE from VB
	switch(m){
		//When the dlg box is created
		case WM_INITDIALOG:{
			//Load & Set the icon
			HICON hIcon = (HICON)LoadImage(hInst,MAKEINTRESOURCE(IDI_ICON),IMAGE_ICON,0,0,LR_DEFAULTCOLOR);
			SendMessage(hw,WM_SETICON,ICON_BIG,(LPARAM)hIcon);
			//Set focus to the load button
			SetFocus(GetDlgItem(hw,IDM_LOAD_ASCII));
			return FALSE;//Prevent the system from setting focus
		}
		//When a command is recieved from the menu/buttons
		case WM_COMMAND:{
			switch(LOWORD(w)){
				case IDM_LOAD_ASCII://Load menu item/button
					Loadmnu_Click(hw);
					return TRUE;//We did process this message
				case IDM_SAVE_DF3://Save menu item/button
					Savemnu_Click(hw);
					return TRUE;
				//Because there is no 'break;' or return statement after the
				//first 'case', control falls into the next 'case'
				case IDCANCEL://X button in the corner or Esc key
				case IDM_QUIT://Quit menu item
					Quitmnu_Click(hw);
					return TRUE;
				case IDM_INPUT_FILE_FORMAT://Input file format menu item
					Infomnu_Click();
					return TRUE;
				case IDM_PROGRAM_INFO://Program info menu item
					Splashmnu_Click();
					return TRUE;
			}//Type of command
		}//Command message
	}//message
	return FALSE;//Did not process the message
}//df3MakerProc